@@ -32,7 +32,7 @@ def _mk_os_yaml(os):
 
 
 def _mk_lock_files(os, in_file):
-    lock_args = ["conda-lock", "--mamba", "-f", str(in_file)]
+    lock_args = ["conda-lock", "--mamba", "--kind", "explicit", "-f", str(in_file)]
     for arg in OS_MAP[os]["build_names"]:
         lock_args.append("-p")
         lock_args.append(arg)
