@@ -24,9 +24,14 @@ from .wind import (
 from .wind_turbine import (
     read_wtg,
     wtg_power,
+    wtg_cp,
     wtg_ct,
+    WindTurbines,
 )
 from .workspace import Workspace
 from .weibull import *
 from .wind_climate import mean_windspeed, power_density
 from . import plot  # Order of imports matters
+
+from .map_conversion.lines2poly import lines2poly
+from .map_conversion.poly2lines import poly2lines
