@@ -1,7 +1,9 @@
 # (c) 2022 DTU Wind Energy
 """Common tools used in vectormap.py and _vectormap_gml.py"""
+import json
 from enum import IntEnum
 from inspect import cleandoc
+from pathlib import Path
 
 import pandas as pd
 
@@ -49,6 +51,13 @@ _LR_COLS = {
 }
 
 
+def _read_map_file_header_to_epsg_table():
+    """Opens the map_file_header_to_epsg.json file and returns it as a dict"""
+
+    with open(Path(__file__).parent / "map_file_header_to_epsg.json", "r") as f:
+        return json.load(f)
+
+
 def _z0_to_landcover(gdf):
     """Converts a roughness to landcover GeoDataFrame.
 
@@ -165,4 +174,4 @@ def explode_gdf(gdf):
     # The explode is needed for converting multipart geometries: wasp only support
     # single part geometries.
     # The reset index is needed so that each single geometry has a unique ID
-    return gdf.explode().reset_index(drop=True)
+    return gdf.explode(index_parts=True).reset_index(drop=True)
