@@ -34,7 +34,7 @@ import xarray as xr
 from lxml import etree
 
 from ._validate import create_validator
-from .metadata import _BWC_ATTRS, update_history, update_var_attrs
+from .metadata import _BWC_ATTRS, _WSBIN_COORD_ATTRS, update_history, update_var_attrs
 from .sector import (
     create_sector_coords,
     create_ws_bin_coords,
@@ -256,7 +256,7 @@ def read_bwc(file, crs=None):
 def combine_bwcs(bwc_list):
     """Combines a list of bwc's into one binned wind climate.
 
-    .. note:: This always an object with a point structure
+    .. note:: The output is always an object with a point structure
 
     Parameters
     ----------
@@ -273,7 +273,7 @@ def combine_bwcs(bwc_list):
     for bwc in bwc_list:
         if not is_point(bwc):
             bwc = to_point(bwc)
-        filled_bwcs.append(_fill_wsbin(bwc, max_bins + 1))
+        filled_bwcs.append(_fill_wsbin(bwc, max_bins))
 
     bwcs = xr.concat(filled_bwcs, "point")
     return update_history(bwcs)
@@ -305,7 +305,7 @@ def _open_tabfile(tab_file, crs=4326):  # pragma: no cover covered_in_public_met
         return map(float, fobj.readline().split())
 
     def _load_tabfile(tab_file, encode="utf_8"):
-        with open(tab_file, "r", newline="\r\n", encoding=encode) as fobj:
+        with open(tab_file, encoding=encode) as fobj:
             header = fobj.readline().strip()
             south_north, west_east, height = _read_floats(fobj)
             tuple_header = tuple(_read_floats(fobj))
@@ -980,7 +980,9 @@ def _fill_wsbin(bwc, nwsbin):  # pragma: no cover covered_in_public_method
     """Expands a binned wind climate to nwsbin number of bins
     This is a useful feature when you want to combine several
     binnen wind climates in one dataset, because that requires that they
-    have the same dimensions.
+    have the same dimensions. The last two bins of the histogram
+    are used to extrapolate the spacing. Up to this point the bins
+    can also be irregular.
 
     Parameters
     ----------
@@ -995,16 +997,23 @@ def _fill_wsbin(bwc, nwsbin):  # pragma: no cover covered_in_public_method
     filled_bwc: :any:`windkit.bwc`
         Binned wind climate with nwsbin number of bins
     """
+    extension_bin = -2  # index of second last bin of histogram
     owsbin = bwc.dims["wsbin"]
-    dws = np.diff(bwc.wsbin[-2:])
-    newbins = np.zeros(nwsbin)
-    newbins[0:owsbin] = bwc.wsbin.values
+    dws = np.diff(bwc.wsbin[extension_bin:])  # get spacing between last two bins
+    newbins = np.zeros(nwsbin)  # create new array with desired length
+    newbins[0:owsbin] = bwc.wsbin.values  # set old bins
     newbins[owsbin:nwsbin] = newbins[owsbin - 1] + dws * np.arange(
         1, nwsbin - owsbin + 1
+    )  #
+    wsceil = np.append(
+        bwc.wsceil, bwc.wsceil[[-1]] + dws * np.arange(1, nwsbin - owsbin + 1)
     )
-    wscenters = 0.5 * (newbins[:-1] + newbins[1:])
-    filled_bwc = bwc.interp(wsbin=newbins[:-1], kwargs={"fill_value": 0.0})
-    filled_bwc["wsceil"] = (("wsbin"), wscenters)
+    wsfloor = np.append(
+        bwc.wsfloor, bwc.wsfloor[[-1]] + dws * np.arange(1, nwsbin - owsbin + 1)
+    )
+    filled_bwc = bwc.interp(wsbin=newbins, kwargs={"fill_value": 0.0})
+    filled_bwc["wsceil"] = (("wsbin"), wsceil, _WSBIN_COORD_ATTRS["wsceil"])
+    filled_bwc["wsfloor"] = (("wsbin"), wsfloor, _WSBIN_COORD_ATTRS["wsfloor"])
 
     return filled_bwc
 
