@@ -25,3 +25,6 @@ class DataStructures(Enum):
     WASP_SITE_COND = "WAsP site conditions"
     LINCOM_GEWC = "LINCOM GEWC"
     TS = "Time Series Wind Climate"
+    WTG = "Wind Turbine Generator"
+    AEP = "Anual Energy Production"
+    WF_FLOW_MAP = "Wind Farm Flow Map"
