@@ -39,6 +39,17 @@ _GEN_COORDS_META = {
     "sector": ALL_VARS_META["sector"],
 }
 
+__all__ = [
+    "empty_wasp_site_factors",
+    "empty_bwc",
+    "empty_wwc",
+    "empty_gwc",
+    "empty_met_fields",
+    "empty_z0meso",
+    "empty_pwc",
+    "empty_wv_count",
+]
+
 
 def _define_std_arrays(output_locs, nsec):
     """Return standard 2D, 3D, and 4D arrays in point format"""
@@ -254,7 +265,15 @@ def empty_wwc(output_locs, nsec=12, not_empty=True, seed=9876538, **kwargs):
     return update_history(ds)
 
 
-def empty_gwc(output_locs, nsec=12, not_empty=True, seed=9876538, **kwargs):
+def empty_gwc(
+    output_locs,
+    nsec=12,
+    not_empty=True,
+    seed=9876538,
+    gen_heights=(50, 100.0, 200.0),
+    gen_roughnesses=(0.0, 0.03, 0.1, 0.4, 1.5),
+    **kwargs,
+):
     """Create empty generalized wind climate dataset.
 
     If not_empty=True, the data variables are filled with meaninful random numbers, e.g.
@@ -272,6 +291,10 @@ def empty_gwc(output_locs, nsec=12, not_empty=True, seed=9876538, **kwargs):
         meaningful data. Defaults to True.
     seed : int
         Seed for the random data, defaults to 9876538.
+    gen_heights : list
+        List of generalized heights to use for coordinates
+    gen_roughnesses : list
+        List of generalized roughnesses to use for coordinates
     kwargs : dict
         Additional arguments.
     Returns
@@ -287,8 +310,8 @@ def empty_gwc(output_locs, nsec=12, not_empty=True, seed=9876538, **kwargs):
         {"A": da_dict["da_4d"], "k": da_dict["da_4d"], "wdfreq": da_dict["da_4d"]},
         attrs=unstack_attrs,
     )
-    gen_rou_coords = np.array([0.0, 0.03, 0.1, 0.4, 1.5], dtype=float)
-    gen_h_coords = np.array([50, 100.0, 200.0], dtype=float)
+    gen_rou_coords = np.array(gen_roughnesses, dtype=float)
+    gen_h_coords = np.array(gen_heights, dtype=float)
     n_gen_rou = len(gen_rou_coords)
     n_gen_h = len(gen_h_coords)
     ds = ds.expand_dims(
@@ -298,6 +321,7 @@ def empty_gwc(output_locs, nsec=12, not_empty=True, seed=9876538, **kwargs):
     )
     ds["A"] = ds["A"].expand_dims({"gen_height": gen_h_coords})
     ds["k"] = ds["k"].expand_dims({"gen_height": gen_h_coords})
+    ds["wdfreq"] = ds["wdfreq"].expand_dims({"gen_height": gen_h_coords})
 
     n_pt = len(ds["point"])
     if not_empty:
@@ -309,8 +333,8 @@ def empty_gwc(output_locs, nsec=12, not_empty=True, seed=9876538, **kwargs):
         # ds['wdfreq']=xr.DataArray(
         # np.random.dirichlet(np.ones(nsec), n_pt).T, ds['wdfreq'].coords, ds['wdfreq'].dims)
         ds["wdfreq"] = xr.DataArray(
-            np.random.dirichlet(np.ones(nsec), (n_gen_rou, n_pt)),
-            dims=("gen_roughness", "point", "sector"),
+            np.random.dirichlet(np.ones(nsec), (n_gen_h, n_gen_rou, n_pt)),
+            dims=("gen_height", "gen_roughness", "point", "sector"),
         )
     ds["gen_roughness"].attrs = {**_GEN_COORDS_META["gen_roughness"]}
     ds["gen_height"].attrs = {**_GEN_COORDS_META["gen_height"]}
