@@ -10,12 +10,17 @@ files.
 """
 
 import re
+import warnings
 from pathlib import Path
 
 import numpy as np
 import xarray as xr
 from lxml import etree
 
+from ._rvea_xml import (
+    _parse_rvea_anemometer_site_details,
+    _parse_rvea_generalised_mean_wind_climate,
+)
 from ._validate import create_validator
 from .metadata import (
     _WEIB_ATTRS,
@@ -34,7 +39,7 @@ _GEN_COORDS_META = {
 
 DATA_VAR_DICT_GWC = {
     "A": ["sector", "gen_height", "gen_roughness"],
-    "wdfreq": ["sector", "gen_roughness"],
+    "wdfreq": ["sector", "gen_height", "gen_roughness"],
     "k": ["sector", "gen_height", "gen_roughness"],
 }
 
@@ -57,7 +62,7 @@ gwc_validate, gwc_validate_wrapper = create_validator(
 
 
 @gwc_validate_wrapper
-def lib_string(gwc):
+def lib_string(gwc, gen_height=None):
     """Generates string representation of gwc dataset.
 
     Parameters
@@ -65,6 +70,10 @@ def lib_string(gwc):
     gwc: xarray.Dataset
         Dataset containing A, k, and wdfreq.
         Dimensions should be ('gen_height', 'gen_roughness', 'sector')
+    gen_height: float
+        Value of gen_height to use for saving to libfile. Since libfiles only allow a
+        single set of wdfreq values, when your data has varying wdfreq values, you need
+        to set this value. It is selected using the .sel selector from xarray.
 
     Returns
     -------
@@ -96,7 +105,6 @@ def lib_string(gwc):
 
         sep = " " if extra else ""
         fmt = "{0:9.%df}" % prec
-
         return sep.join([fmt.format(i) for i in dat])
 
     def _to_string(node):
@@ -117,10 +125,13 @@ def lib_string(gwc):
         nrough = node.dims["gen_roughness"]
         nhgt = node.dims["gen_height"]
         nsec = node.dims["sector"]
-        node = node.transpose(..., "gen_height", "gen_roughness", "sector")
         node = reproject(node, 4326)
+        # Extract numpy arrays to speed up the processing
+        A = node.A.values
+        k = node.k.values
+        wdfreq = node.wdfreq.values
         string = ""
-        lend = "\n"  # Always use windows line ending
+        newline = "\n"
 
         try:
             height = node.height
@@ -133,59 +144,79 @@ def lib_string(gwc):
         )
         string += (
             header_without_coord
-            + f"<coordinates>{float(node.west_east)},{float(node.south_north)},{float(height)}</coordinates>{lend}"
+            + f"<coordinates>{float(node.west_east)},{float(node.south_north)},{float(height)}</coordinates>{newline}"
         )
 
         # Write the dimensions nz0,nz,nsec
-        string += " ".join([str(i) for i in (nrough, nhgt, nsec)]) + lend
+        string += " ".join([str(i) for i in (nrough, nhgt, nsec)]) + newline
 
         # Write the roughness classes
-        string += _fmt_floats(node.gen_roughness.values, 3, True) + lend
+        string += _fmt_floats(node.gen_roughness.values, 3, True) + newline
 
         # Write the heights
-        string += _fmt_floats(node.gen_height.values, 1, True) + lend
+        string += _fmt_floats(node.gen_height.values, 1, True) + newline
 
         # Write the data arrays
-        for i, _ in enumerate(node.gen_roughness.values):
-
-            # sectorwise frequency
-            string += _fmt_floats(node.wdfreq.isel(gen_roughness=i).values, 2) + lend
-
-            for j, _ in enumerate(node.gen_height.values):
-
+        for i in range(nrough):
+            # sectorwise frequency in percent
+            string += _fmt_floats(100.0 * wdfreq[i, ...], 2) + newline
+            for j in range(nhgt):
                 # Sectorwise A's
-                string += (
-                    _fmt_floats(node.A.isel(gen_height=j, gen_roughness=i).values, 2)
-                    + lend
-                )
-
+                string += _fmt_floats(A[j, i, ...], 2) + newline
                 # Sectorwise k's
-                string += (
-                    _fmt_floats(node.k.isel(gen_height=j, gen_roughness=i).values)
-                    + lend
-                )
+                string += _fmt_floats(k[j, i, ...]) + newline
 
         return string
 
+    dims_order = ["gen_height", "gen_roughness", "sector"]
+
+    gwc = gwc.copy()
+    gwc = gwc.transpose(*dims_order, ...)
+
+    # if wdfreq varues with height we require users to choose the height
+    if _wdfreq_constant_with_gen_height(gwc):
+        gwc["wdfreq"] = gwc["wdfreq"].isel(gen_height=0)
+    else:
+        if gen_height is None:
+            raise ValueError(
+                "Lib files do not support different 'wdfreq' values for different heights. Please specify the 'gen_height' for the 'wdfreq' values that you wish to use for all levels."
+            )
+        else:
+            gwc["wdfreq"] = gwc["wdfreq"].sel(gen_height=gen_height, method="nearest")
+
     if gwc.squeeze().A.ndim == 3:
         return _to_string(gwc.squeeze())
 
-    dims_extra = [
-        d for d in gwc.A.dims if d not in ["gen_height", "gen_roughness", "sector"]
-    ]
-    stacked = gwc.stack(point=dims_extra)
-
+    dims_extra = [d for d in gwc.A.dims if d not in dims_order]
+    stacked = gwc.stack(point=dims_extra).transpose(*dims_order, "point")
     # Get numbers of sectors, roughness classes and
     strings = []
     for ipt in range(stacked.dims["point"]):
+        # Need the slice to keep the point dimension as multi-index
         node = stacked.isel(point=slice(ipt, ipt + 1)).reset_index("point").squeeze()
         strings.append(_to_string(node))
 
     return strings
 
 
+def _wdfreq_constant_with_gen_height(gwc):
+    """Check if 'wdfreq' is constant with 'gen_height' dimension.
+
+    Parameters
+    ----------
+    gwc : xarray.Dataset
+        Generalized wind climate dataset.
+
+    Returns
+    -------
+    bool
+        True if 'wdfreq' is constant with 'gen_height' dimension across whole dataset.
+    """
+    return np.isclose(gwc["wdfreq"].diff(dim="gen_height").max(), 0.0).all()
+
+
 @gwc_validate_wrapper
-def to_libfile(gwc, path=None):
+def to_libfile(gwc, path=None, gen_height=None):
     """Creates lib-style ascii file from gwc dataset
 
     Parameters
@@ -195,11 +226,19 @@ def to_libfile(gwc, path=None):
     path : str
         dir or file path for storing lib file
         Default value set to the current working directory.
+    gen_height: float
+        Value of gen_height to use for saving to libfile. Since libfiles only allow a
+        single set of wdfreq values, when your data has varying wdfreq values, you need
+        to set this value. It is selected using the .sel selector from xarray.
+
     """
 
+    gwc = gwc.copy()
+
     def _write(node, fpath):
+        # Set newline explicitly in string not in open.
         with open(fpath, "w", newline="\r\n") as fobj:
-            fobj.write(lib_string(node))
+            fobj.write(lib_string(node, gen_height=gen_height))
 
     if path is None:
         path = Path.cwd()
@@ -237,7 +276,9 @@ def to_libfile(gwc, path=None):
     return
 
 
-def read_gwc(file_or_obj, crs=None, coords=None):
+def read_gwc(
+    file_or_obj, crs=None, west_east=None, south_north=None, height=None, header=None
+):
     """Create gwc xarray.Dataset from file.
 
     Parameters
@@ -249,33 +290,58 @@ def read_gwc(file_or_obj, crs=None, coords=None):
     crs : int, dict, str or pyproj.crs.CRS
         Value to initialize `pyproj.crs.CRS`
         Defaults to 4326 (Lat-Lon on the WGS84 geodetic datum). for .lib and .gwc.
-    coords : tuple
-        Coordinates of the .lib file. (west_east, south_north, height)
-        By default this will try to read a coordinates field from the header.
+    west_east : float or None, optional
+        West-East coordinate of the GWC grid, by default None, which will attempt to
+        read from the file.
+    south_north : float or None, optional
+        South-North coordinate of the GWC grid, by default None, which will attempt to
+        read from the file.
+    height : float or None, optional
+        Height coordinate of the GWC grid, by default None, which will attempt to
+        read from the file.
+    header : str or None, optional
+        Header string to use for .lib and .gwc files, by default None, which will
+        attempt to read from the file.
 
     Returns
     -------
     xarray.Dataset
         Generalized wind climate dataset.
+
+    Raises
+    ------
+    ValueError
+        If the file extension is not recognized.
+    ValueError
+        If the requested crs does not match the dataset crs.
+
     """
     file_or_obj = Path(file_or_obj)
     ext = file_or_obj.suffix
 
     if ext == ".lib":
-        if crs is None:
-            crs = 4326
-        ds = _open_libfile(file_or_obj, crs, coords)
+        ds = _read_lib_file(
+            file_or_obj,
+            crs=crs,
+            west_east=west_east,
+            south_north=south_north,
+            height=height,
+            header=header,
+        )
     elif ext in [".gwc"]:
-        if crs is None:
-            crs = 4326
-        tree = etree.parse(str(file_or_obj))
-        ds = _parse_gwc(tree.getroot(), crs)
+        ds = _read_gwc_file(
+            file_or_obj,
+            crs=crs,
+            west_east=west_east,
+            south_north=south_north,
+            height=height,
+            header=header,
+        )
     elif ext in [".nc"]:
         ds = xr.open_dataset(file_or_obj)
         gwc_validate(ds)
         if crs is not None and not crs_are_equal(ds, crs):
             raise ValueError(f"Requested crs does not match dataset crs")
-        ds
     else:
         raise ValueError(
             f"Unable to detect type of gwc file {file_or_obj} with extension {ext}."
@@ -285,8 +351,10 @@ def read_gwc(file_or_obj, crs=None, coords=None):
     return update_history(ds)
 
 
-def _open_libfile(lib_file, crs=4326, xyz_coords=None):
-    """
+def _read_lib_file(
+    lib_file, crs=None, west_east=None, south_north=None, height=None, header=None
+):
+    r"""
     Create GenWindClimate object from WAsP .lib file
 
     Parameters
@@ -296,15 +364,30 @@ def _open_libfile(lib_file, crs=4326, xyz_coords=None):
     crs : int, dict, str or pyproj.crs.CRS
         Value to initialize `pyproj.crs.CRS`
         Defaults to 4326 (Lat-Lon on the WGS84 geodetic datum)
-    xyz_coords : tuple
-        Coordinates of of the .lib file. (west_east, south_north, height)
-        By default this will try to read a coordinates field from the header
+    west_east : float or None, optional
+        West-East coordinate of the GWC grid, by default None, which will attempt to
+        read from the file.
+    south_north : float or None, optional
+        South-North coordinate of the GWC grid, by default None, which will attempt to
+        read from the file.
+    height : float or None, optional
+        Height coordinate of the GWC grid, by default None, which will attempt to
+        read from the file.
+    header : str or None, optional
+        Header string to use for .lib and .gwc files, by default None, which will
+        attempt to read from the file.
 
     Returns
     -------
     xr.DataSet
         xarray DataSet that is formatted to match the gwc description
 
+    Raises
+    ------
+    ValueError
+        If coordinates are not present in the file and are not provided as
+        arguments.
+
     Notes
     -----
     Automatically adds lat, lon coords if present inside
@@ -342,18 +425,17 @@ def _open_libfile(lib_file, crs=4326, xyz_coords=None):
         return [np.int32(i) for i in f.readline().strip().split()]
 
     # Open libfile
-    with open(lib_file, newline="\r\n") as f:
-
+    with open(lib_file) as f:
         # Read header information one line at a time
         desc = f.readline().strip()  # File Description
         nrough, nhgt, nsec = _read_int_(f)  # dimensions
-        roughnesses = _read_float_(f)  # Roughness classes
-        heights = _read_float_(f)  # heights
+        gen_roughness = _read_float_(f)  # Roughness classes
+        gen_height = _read_float_(f)  # heights
 
         # Initialize arrays
-        freq = np.zeros([nrough, nsec], dtype="f4", order="F")
-        k = np.zeros([nhgt, nrough, nsec], dtype="f4", order="F")
-        A = np.zeros([nhgt, nrough, nsec], dtype="f4")
+        wdfreq = np.zeros([nsec, nrough], dtype="f4", order="F")
+        k = np.zeros([nsec, nhgt, nrough], dtype="f4", order="F")
+        A = np.zeros([nsec, nhgt, nrough], dtype="f4")
 
         ##################################################################
         # The remainder of the file is made up of rows with nsec columns.
@@ -361,50 +443,54 @@ def _open_libfile(lib_file, crs=4326, xyz_coords=None):
         # A & k rows for each height.
         ##################################################################
         # Loop over roughness classes to read frequency line
-        for i, dummy in enumerate(roughnesses):
-            freq[i, :] = _read_float_(f)
+        for i, dummy in enumerate(gen_roughness):
+            wdfreq[:, i] = _read_float_(f)
             # Loop over heights to read in all A & k values
-            for j, dummy in enumerate(heights):
-                A[j, i, :] = _read_float_(f)
-                k[j, i, :] = _read_float_(f)
+            for j, dummy in enumerate(gen_height):
+                A[:, j, i] = _read_float_(f)
+                k[:, j, i] = _read_float_(f)
 
-    ak_dims = ("gen_height", "gen_roughness", "sector", "point")
-    f_dims = ("gen_roughness", "sector", "point")
+    if crs is None:
+        crs = "EPSG:4326"
 
     # Find the coordinates if they aren't provided
-    if xyz_coords is None:
+    if all(c is None for c in [west_east, south_north, height]):
         # Find the coordinates
         latlons = re.search("<coordinates>(.*)</coordinates>", desc)
         if latlons:
-            lon, lat, height = map(np.float32, latlons.group(1).split(","))
+            west_east, south_north, height = map(
+                np.float32, latlons.group(1).split(",")
+            )
         else:
             raise ValueError(
-                f"Coordinates array not found in file {lib_file}, "
-                + "please set coords argument."
+                f"Coordinates not found in {lib_file}, "
+                + "please set 'west_east', 'south_north', and 'height' explicitly. "
+                + "These values should ge in the projection given by the 'crs' argument,"
+                + " which defaults to the WGS84 projection."
             )
-    else:  # Use user provided coordinates
-        lon, lat, height = xyz_coords
 
-    # Create dataset
-    ds = create_dataset(lon, lat, height, crs).drop_vars("output")
-    ds.attrs = {"wasp_header": desc}
+    if header is None:
+        if len(desc) > 0:
+            header = desc
+        else:
+            header = ""
 
-    # Add variables
-    ds["A"] = (ak_dims, np.reshape(A, A.shape + (1,)))
-    ds["k"] = (ak_dims, np.reshape(k, k.shape + (1,)))
-    ds["wdfreq"] = (f_dims, np.reshape(freq, freq.shape + (1,)))
+    # Add height to wdfreq
+    wdfreq = np.tile(wdfreq[:, np.newaxis, :], (1, nhgt, 1))
 
-    # Add coordinates
-    ds = ds.assign_coords(
-        {
-            **create_coords(heights, "gen_height", _GEN_COORDS_META).coords,
-            **create_coords(roughnesses, "gen_roughness", _GEN_COORDS_META).coords,
-            **create_sector_coords(nsec).coords,
-        }
+    return _weibull_to_dataset(
+        wdfreq,
+        A,
+        k,
+        gen_roughness,
+        gen_height,
+        south_north,
+        west_east,
+        height,
+        crs,
+        wasp_header=header,
     )
 
-    return ds
-
 
 def _weibull_to_dataset(
     wdfreq,
@@ -423,24 +509,29 @@ def _weibull_to_dataset(
 
     Parameters
     ----------
-    wdfreq : numpy
-        Wind direction frequency by sector and generalized roughnesses
+    wdfreq : 1-D sequence of floats
+        Wind direction frequencies. Dimensions=[sector, gen_height, gen_roughnness]
     A : numpy
-        Weibull A parameter by sector and generalized roughness and height
+        Weibull A parameters.  Dimensions=[sector, gen_height, gen_roughnness]
     k : numpy
-        Weibull k parameter by sector and generalized roughness and height
+        Weibull k parameters.  Dimensions=[sector, gen_height, gen_roughnness]
     gen_roughness : numpy
         Array of generalized roughnesses
     gen_height : numpy
         Array of generalized heights
-    south_north: float64
-        Coordinate value in y-direction
-    west_east: float64
-        Coordinate value in x-direction
-    height: float64
-        Height above ground
+    west_east : float or None, optional
+        West-East coordinate of the GWC grid, by default None, which will attempt to
+        read from the file.
+    south_north : float or None, optional
+        South-North coordinate of the GWC grid, by default None, which will attempt to
+        read from the file.
+    height : float or None, optional
+        Height coordinate of the GWC grid, by default None, which will attempt to
+        read from the file.
     crs : int, dict, str or pyproj.crs.CRS
         Value to initialize `pyproj.crs.CRS`
+        Defaults to 4326 (Lat-Lon on the WGS84 geodetic datum)
+
     kwargs : dict, optional
         Other key-word arguments are added as attributes to the dataset.
 
@@ -450,22 +541,19 @@ def _weibull_to_dataset(
         WindKit GWC dataset
     """
 
-    nsec, _ = wdfreq.shape
+    nsec, _, _ = wdfreq.shape
 
     na = np.newaxis
 
-    wdfreq /= np.sum(wdfreq)
-
     # Create dataset
     ds = create_dataset(west_east, south_north, height, crs).drop_vars("output")
     ds.attrs = kwargs
 
     # Add variables
-    ak_dims = ("sector", "gen_height", "gen_roughness", "point")
-    f_dims = ("sector", "gen_roughness", "point")
-    ds["A"] = (ak_dims, A[:, :, :, na])
-    ds["k"] = (ak_dims, k[:, :, :, na])
-    ds["wdfreq"] = (f_dims, wdfreq[:, :, na])
+    dims = ("sector", "gen_height", "gen_roughness", "point")
+    ds["A"] = (dims, A[..., na])
+    ds["k"] = (dims, k[..., na])
+    ds["wdfreq"] = (dims, wdfreq[..., na])
 
     # Add coordinates
     ds = ds.assign_coords(
@@ -476,75 +564,121 @@ def _weibull_to_dataset(
         }
     )
 
+    ds["wdfreq"] = ds["wdfreq"] / ds["wdfreq"].sum(dim="sector")
+
     return ds.transpose("gen_height", "gen_roughness", "sector", "point")
 
 
-def _parse_gwc(gwc, crs=4326):
-    """
-    Parses an gwc XML file into a gwc object
+def _read_gwc_file(
+    gwc_file,
+    crs=None,
+    west_east=None,
+    south_north=None,
+    height=None,
+    header=None,
+):
+    """Read Generalized Wind Climate from XML-based .gwc file.
 
     Parameters
     ----------
-    gwc : xml tree
-        An XML element loaded by lxml
+    gwc_file : str, pathlib.Path
+        input file
     crs : int, dict, str or pyproj.crs.CRS
         Value to initialize `pyproj.crs.CRS`
-        Defaults to 4326 (Lat-Lon/WGS84 geodetic datum)
+        Defaults to 4326 (Lat-Lon on the WGS84 geodetic datum)
+    west_east : float or None, optional
+        West-East coordinate of the GWC grid, by default None, which will attempt to
+        read from the file.
+    south_north : float or None, optional
+        South-North coordinate of the GWC grid, by default None, which will attempt to
+        read from the file.
+    height : float or None, optional
+        Height coordinate of the GWC grid, by default None, which will attempt to
+        read from the file.
+    header : str or None, optional
+        Header string to use for .lib and .gwc files, by default None, which will
+        attempt to read from the file.
 
     Returns
     -------
-    xr.DataSet
-        xarray DataSet that is formatted to match the gwc description
+    xarray.Dataset
+        WindKit GWC dataset
+
+    Raises
+    ------
+    ValueError
+        If no RveaGeneralisedMeanWindClimate is found in the file.
+    ValueError
+        If coordinates are not found in the file and are not provided explicitly.
+
+    Warns
+    -----
+    UserWarning
+        If 'height' is not found in the file and are not provided explicitly.
+        It will be set to 0.0
+
     """
-    # Get site info
-    site_info = gwc.xpath("//RveaAnemometerSiteDetails")[0].attrib
-    height = float(site_info["HeightAGL"])
-    header = site_info["Description"]
-    lat = float(site_info["LatitudeDegrees"])
-    lon = float(site_info["LongitudeDegrees"])
 
-    # Subset GWC to only GWC information (drop children)
-    gwc = gwc.xpath("./MemberData/RveaGeneralisedMeanWindClimate")[0]
+    if crs is None:
+        crs = "EPSG:4326"
 
-    # Get main dimensions
-    num_sec = int(gwc.attrib["CountOfSectors"])
-    num_gen_height = int(gwc.attrib["CountOfReferenceHeights"])
-    num_gen_roughness = int(gwc.attrib["CountOfReferenceRoughnessLengths"])
+    tree = etree.parse(str(gwc_file))
+    root = tree.getroot()
 
-    # Create arrays
-    wdfreq = np.zeros((num_sec, num_gen_roughness))
-    A = np.zeros((num_sec, num_gen_height, num_gen_roughness))
-    k = np.zeros((num_sec, num_gen_height, num_gen_roughness))
-    cen_angle = np.zeros(num_sec)
-    gen_roughness = np.zeros(num_gen_roughness)
-    gen_height = np.zeros(num_gen_height)
+    descendants = list(e.tag for e in root.getiterator())
 
-    # Get wind speed histogram
-    for wawwr in gwc.xpath("WindAtlasWeibullWindRose"):
-        # Get array info information
+    if root.tag == "RveaGeneralisedMeanWindClimate":
+        gwc_tree = root
+    elif "RveaGeneralisedMeanWindClimate" in descendants:
+        gwc_tree = tree.find(".//RveaGeneralisedMeanWindClimate")
+    else:
+        raise ValueError(f"No RveaGeneralisedMeanWindClimate found in '{gwc_file}'")
 
-        rough_no = int(wawwr.attrib["RoughnessLengthNumber"]) - 1
-        height_no = int(wawwr.attrib["ReferenceHeightNumber"]) - 1
+    gwc_data = _parse_rvea_generalised_mean_wind_climate(gwc_tree)
 
-        gen_height[height_no] = float(wawwr.attrib["ReferenceHeight"])
-        gen_roughness[rough_no] = float(wawwr.attrib["RoughnessLength"])
-        for ww in wawwr.xpath("WeibullWind"):
-            sec = int(ww.attrib["Index"]) - 1
-            cen_angle[sec] = float(ww.attrib["CentreAngleDegrees"])
-            A[sec, height_no, rough_no] = float(ww.attrib["WeibullA"])
-            k[sec, height_no, rough_no] = float(ww.attrib["WeibullK"])
-            wdfreq[sec, rough_no] = float(ww.attrib["SectorFrequency"])
+    if "RveaAnemometerSiteDetails" in descendants:
+        site_data = _parse_rvea_anemometer_site_details(
+            tree.find(".//RveaAnemometerSiteDetails")
+        )
+    else:
+        site_data = {}
 
-    kwargs = {
-        "gen_roughness": gen_roughness,
-        "gen_height": gen_height,
-        "south_north": lat,
-        "west_east": lon,
-        "height": height,
-        "wasp_header": header,
-        "crs": crs,
-    }
-    return _weibull_to_dataset(wdfreq, A, k, **kwargs)
+    if west_east is None:
+        west_east = gwc_data.get("longitude", site_data.get("longitude", None))
+
+    if south_north is None:
+        south_north = gwc_data.get("latitude", site_data.get("latitude", None))
+
+    if height is None:
+        height = gwc_data.get("height", site_data.get("height", None))
+
+    if header == None:
+        header = gwc_data.get("header", site_data.get("header", ""))
+
+    if west_east is None or south_north is None:
+        raise ValueError(
+            "'west_east' or 'south_north' coordinate not found in file or provided as argument. "
+            + "Please set 'west_east' and 'south_north' explicitly."
+        )
+
+    if height is None:
+        height = 0.0
+        warnings.warn(
+            "No height found in file or provided as argument, using 0 m. Set 'height' explicitly to avoid this warning.",
+        )
+
+    return _weibull_to_dataset(
+        wdfreq=gwc_data["wdfreq"],
+        A=gwc_data["A"],
+        k=gwc_data["k"],
+        gen_roughness=gwc_data["gen_roughness"],
+        gen_height=gwc_data["gen_height"],
+        west_east=west_east,
+        south_north=south_north,
+        height=height,
+        crs=crs,
+        wasp_header=header,
+    )
 
 
 @gwc_validate_wrapper
