@@ -2,7 +2,7 @@ import inspect
 
 # Check importability of geopandas
 try:
-    import geopandas as _
+    import geopandas as _  # nopycln: import
 
     HAS_GEOPANDAS = True
 except (ModuleNotFoundError, ImportError):
@@ -39,7 +39,7 @@ def is_GeoSeries(obj):
 
 # Check importability of rasterio
 try:
-    import rasterio as _
+    import rasterio as _  # nopycln: import
 
     has_rasterio = True
 except (ModuleNotFoundError, ImportError):
