@@ -306,6 +306,54 @@ _WEIB_ATTRS = {
     "object_type": DataStructures.WEIB.value,
 }
 
+_WTG_ATTRS = {
+    "wind_speed": ALL_VARS_META["wind_speed"],
+    "mode": ALL_VARS_META["wtg_mode"],
+    "power_output": ALL_VARS_META["power_output"],
+    "thrust_coefficient": ALL_VARS_META["thrust_coefficient"],
+    "air_density": ALL_VARS_META["air_density"],
+    "stationary_thrust_coefficient": ALL_VARS_META["stationary_thrust_coefficient"],
+    "wind_speed_cutin": ALL_VARS_META["wind_speed_cutin"],
+    "wind_speed_cutout": ALL_VARS_META["wind_speed_cutout"],
+    "rated_power": ALL_VARS_META["rated_power"],
+    "name": ALL_VARS_META["wtg_model"],
+    "rotor_diameter": ALL_VARS_META["rotor_diameter"],
+    "hub_height": ALL_VARS_META["hub_height"],
+    "object_type": DataStructures.WTG.value,
+}
+
+# Data structure returned by aep calculations
+_AEP_ATTRS = {
+    "gross_AEP": ALL_VARS_META["gross_aep"],
+    "gross_AEP_sector": ALL_VARS_META["gross_aep_sector"],
+    "potential_AEP": ALL_VARS_META["potential_aep"],
+    "potential_AEP_sector": ALL_VARS_META["potential_aep_sector"],
+    "object_type": DataStructures.AEP.value,
+}
+# Data structure for a wind farm flow map
+_WF_FLOW_MAP_ATTRS = {
+    "potential_AEP_sector": ALL_VARS_META["potential_aep_sector"],
+    "gross_AEP_sector": ALL_VARS_META["gross_aep_sector"],
+    "AEP_deficit_sector": ALL_VARS_META["aep_deficit_sector"],
+    "wspd_sector": ALL_VARS_META["wind_speed"],  # Is this right?
+    "wspd_eff_sector": ALL_VARS_META["wind_speed_effective"],  # Is this right?
+    "wspd_deficit_sector": ALL_VARS_META["wind_speed_deficit"],  # Is this right?
+    "turbulence_intensity_eff_sector": ALL_VARS_META[
+        "turbulence_intensity_effective_sector"
+    ],  # there is a similar
+    "wdfreq": ALL_VARS_META["wdfreq"],
+    "potential_AEP": ALL_VARS_META["potential_aep"],
+    "gross_AEP": ALL_VARS_META["potential_aep"],
+    "AEP_deficit": ALL_VARS_META["aep_deficit"],
+    "wspd": ALL_VARS_META["wind_speed"],  # Is this right?
+    "wspd_eff": ALL_VARS_META["wind_speed_effective"],  # Is this right?
+    "wspd_deficit": ALL_VARS_META["wind_speed_deficit"],  # Is this right?
+    "turbulence_intensity_eff": ALL_VARS_META[
+        "turbulence_intensity_effective"
+    ],  # there is a siilar
+    "object_type": DataStructures.WF_FLOW_MAP.value,
+}
+
 
 def _update_local_attrs(da, var_dict):
     """Updates data varaible attributes
