@@ -2,7 +2,7 @@
 """
 Generic default colormap module
 """
-from ._helpers import HAS_MATPLOTLIB, requires_matplotlib
+from ._helpers import HAS_MATPLOTLIB
 
 if HAS_MATPLOTLIB:
     import matplotlib as mpl
@@ -17,7 +17,14 @@ if HAS_MATPLOTLIB:
 
     colormaps_dict = {
         "z0meso": mpl.colors.LinearSegmentedColormap.from_list(
-            "", ["#2A479E", "#0DCF69", "#F9FA96", "#A8906A", "#006600",]
+            "",
+            [
+                "#2A479E",
+                "#0DCF69",
+                "#F9FA96",
+                "#A8906A",
+                "#006600",
+            ],
         ),
         "user_def_speedups": "coolwarm",
         "orographic_speedups": "coolwarm",
