@@ -14,6 +14,8 @@ Supported objects include:
    4. **raster** (..., *south_north*, *west_east*) this is an internal structure that behaves like a 2D **cuboid**
 """
 from ._cuboid import to_cuboid
+from ._interpolation import interpolate_to_grid
+from ._latitude import get_latitude
 
 # nopycln: file
 from ._point import to_point, to_stacked_point
@@ -26,6 +28,7 @@ from ._struct import (
     is_stacked_point,
     is_vertical,
 )
+from ._utm import *
 from .decorators import stack_then_unstack
 from .spatial import *
 
@@ -39,4 +42,6 @@ __all__ = [
     "clip",
     "warp",
     "create_dataset",
+    "get_latitude",
+    "interpolate_to_grid",
 ]
