@@ -87,6 +87,12 @@ def add_crs(obj, crs):
         obj.coords["crs"] = np.byte(0)
 
         pyproj_crs = pyproj.CRS.from_user_input(crs)
+        # Check if user input matches an EPSG code, and use that if it does
+        try:
+            pyproj_crs = pyproj.CRS.from_epsg(pyproj_crs.to_epsg())
+        except pyproj.exceptions.CRSError:
+            pass
+
         crs_cf_dict = pyproj_crs.to_cf()
         crs_cf_dict["long_name"] = "CRS definition"
         # Needed for GDAL compliance
@@ -132,4 +138,4 @@ def crs_are_equal(obj_a, obj_b):
             crs = get_crs(obj)
         crs_list.append(crs)
     crs_a, crs_b = crs_list
-    return crs_a.is_exact_same(crs_b)
+    return crs_a.equals(crs_b)
