@@ -31,10 +31,11 @@ vertical (partly public)
         height: ('height',)
 
 stacked_point (private)
-    dimensions: (..., 'stacked_point')
+    dimensions: (..., 'height', 'stacked_point')
     coords:
         south_north: ('stacked_point',)
         west_east: ('stacked_point',)
+        height: ('height',)
 
 """
 
@@ -45,21 +46,21 @@ _WINDKIT_STACKED_POINT_DIM = "stacked_point"
 _WINDKIT_POINT_DIM = "point"
 
 
-def _get_spatial_dims():
+def _xy_dims():
     return _WINDKIT_X_DIM, _WINDKIT_Y_DIM
 
 
-def _get_vertical_dim():
+def _vertical_dim():
     return _WINDKIT_Z_DIM
 
 
-def _get_point_dim():
+def _point_dim():
     return _WINDKIT_POINT_DIM
 
 
-def _get_stacked_point_dim():
+def _stacked_point_dim():
     return _WINDKIT_STACKED_POINT_DIM
 
 
-def _get_cuboid_dims():
+def _cuboid_dims():
     return _WINDKIT_X_DIM, _WINDKIT_Y_DIM, _WINDKIT_Z_DIM
