@@ -360,7 +360,7 @@ def _mask_raster(
     transform_ = _transform(obj)
     crs_mask = pyproj.CRS.from_user_input(mask.crs)
 
-    if not crs_obj.is_exact_same(crs_mask):
+    if not crs_obj.equals(crs_mask):
         raise ValueError(
             "Raster CRS and mask CRS are not identical!" + " please reproject first!"
         )
@@ -424,7 +424,7 @@ def _clip_to_bbox_raster(obj, bbox):  # pragma: no cover covered_in_public_metho
 
     if isinstance(bbox, (BBox)) or is_GeoDataFrame(bbox) or is_GeoSeries(bbox):
         crs_mask = pyproj.CRS.from_user_input(bbox.crs)
-        if not crs_raster.is_exact_same(crs_mask):
+        if not crs_raster.equals(crs_mask):
             raise ValueError(
                 "Raster CRS and mask CRS are not identical!"
                 + " please reproject first!"
@@ -533,6 +533,18 @@ def _can_be_raster(west_east, south_north, thresh=1e-9):
     return True
 
 
+def _has_raster_dims(west_east, south_north):
+    """Test if an array of coordinate values can be a raster and do not
+    have duplicates.
+    """
+    if (west_east.size != np.unique(west_east).size) or (
+        south_north.size != np.unique(south_north).size
+    ):
+        return False
+    else:
+        return _can_be_raster(west_east, south_north)
+
+
 def to_raster(obj, ignore_raster_check=False):
     """Converts a point based object to a raster based object
 
