@@ -17,14 +17,14 @@ from ._crs import add_crs, crs_are_equal, get_crs
 from ._dimensions import _point_dim, _stacked_point_dim, _vertical_dim, _xy_dims
 from ._point import _mask_point, is_point, is_stacked_point, to_point, to_stacked_point
 from ._raster import (
-    _can_be_raster,
     _clip_to_bbox_raster,
+    _has_raster_dims,
     _mask_raster,
     _warp_raster,
     is_raster,
     to_raster,
 )
-from ._struct import get_spatial_struct
+from ._struct import _from_scalar, get_spatial_struct, is_cuboid
 from ._vector import _clip_vector
 from ._vertical import has_height_coord, has_height_dim
 
@@ -35,6 +35,7 @@ __all__ = [
     "clip",
     "crs_are_equal",
     "create_dataset",
+    "count_spatial_points",
     "get_crs",
     "mask",
     "reproject",
@@ -47,6 +48,40 @@ __all__ = [
 logger = logging.getLogger(__name__)
 
 
+def count_spatial_points(obj):
+    """Get the number of spatial points for a dataset or DataArray
+
+    Parameters
+    ----------
+    obj: xarray.Dataset, xarray.DataArray
+        WindKit xarray dataset or dataarray containing spatial dimensions.
+
+    Raises
+    ------
+    ValueError
+        Undetectable spatial structure
+    """
+    obj = _from_scalar(obj)
+    dims = obj.sizes
+
+    # Get dimension names
+    xdim, ydim = _xy_dims()
+    pt_dim = _point_dim()
+    vert_dim = _vertical_dim()
+    spt_dim = _stacked_point_dim()
+
+    if is_cuboid(obj):
+        return dims[vert_dim] * dims[ydim] * dims[xdim]
+    elif is_raster(obj):
+        return dims[ydim] * dims[xdim]
+    elif is_stacked_point(obj):
+        return dims[vert_dim] * dims[spt_dim]
+    elif is_point(obj):
+        return dims[pt_dim]
+    else:
+        raise ValueError("Unknown data structure cannot count points.")
+
+
 def are_spatially_equal(obj_a, obj_b):
     """Checks that the spatial points are equivalent for both datasets
 
@@ -137,7 +172,7 @@ def create_dataset(west_east, south_north, height, crs, struct="auto", thresh=1e
 
     if struct.lower() == "auto":
 
-        if _can_be_raster(x, y):
+        if _has_raster_dims(x, y):
             struct = "cuboid"
         elif (y.size == x.size) and (x.size != z.size):
             struct = "stacked_point"
@@ -154,7 +189,7 @@ def create_dataset(west_east, south_north, height, crs, struct="auto", thresh=1e
 
     if struct.lower() == "cuboid":
 
-        if not _can_be_raster(x, y):
+        if not _has_raster_dims(x, y):
             raise ValueError("Data cannot be converted to raster or cuboid dataset.")
 
         # Create spatial data arrays
@@ -302,9 +337,7 @@ def reproject(obj, to_crs, copy=True):
     to_crs = pyproj.CRS.from_user_input(to_crs)
 
     # Reproject dimensions
-    transformer = pyproj.Transformer.from_crs(
-        from_crs, to_crs, skip_equivalent=True, always_xy=True
-    )
+    transformer = pyproj.Transformer.from_crs(from_crs, to_crs, always_xy=True)
 
     struct = get_spatial_struct(obj)
 
@@ -324,6 +357,7 @@ def reproject(obj, to_crs, copy=True):
     x_new = _replace_close(x_new)
     y_new = _replace_close(y_new)
 
+    obj = obj.drop_vars([x_dim, y_dim])
     obj = obj.assign_coords({x_dim: ((struct), x_new), y_dim: ((struct), y_new)})
 
     obj = add_crs(obj, to_crs)
@@ -739,5 +773,5 @@ def nearest_points(
             .drop_vars("point_tmp")
         )
         if not return_rank:
-            nearest = nearest.drop("rank")
+            nearest = nearest.drop_vars("rank")
         return update_history(nearest)
