@@ -14,7 +14,6 @@ elevation maps, and .gml for land-cover maps.
 """
 import argparse
 import logging
-import re
 from collections import defaultdict
 from inspect import cleandoc
 from pathlib import Path
@@ -26,10 +25,15 @@ from shapely.geometry import LineString, Polygon
 
 from ._vectormap_gml import read_vector_map_gml, write_vectormap_gml
 from ._vectormap_helpers import *
-from ._vectormap_helpers import _MAP_TYPE_CODES, _is_elev, _is_lc, _z0_to_landcover
+from ._vectormap_helpers import (
+    _MAP_TYPE_CODES,
+    _is_elev,
+    _is_lc,
+    _read_map_file_header_to_epsg_table,
+    _z0_to_landcover,
+)
 from .landcover import LandCoverTable
 from .spatial import BBox
-from .spatial._utm import _utmzone2epsg
 
 logger = logging.getLogger("__name__")
 
@@ -70,6 +74,7 @@ def _read_vector_map_combo(filename, crs, map_type):
     got_header_line = False
     current_point_count = 0
     ltype = -999
+    n_pts_geom = None
 
     data = defaultdict(list)
 
@@ -97,7 +102,6 @@ def _read_vector_map_combo(filename, crs, map_type):
         raise ValueError("""'crs' cannot be detected from file""")
 
     with open(filename, "r", newline="\r\n") as fobj:
-
         # Read 4 header lines into a single string
         for _ in range(4):
             metadata += fobj.readline()
@@ -108,7 +112,6 @@ def _read_vector_map_combo(filename, crs, map_type):
 
             # Read line header
             if got_header_line is False:
-
                 z0_left, z0_right, elev = np.nan, np.nan, np.nan
 
                 # Check for empty lines
@@ -140,6 +143,11 @@ def _read_vector_map_combo(filename, crs, map_type):
                     z0_left, z0_right, elev, n_pts_geom = elements
                     ltype = 2
 
+                # Combination line with displacement height
+                if len(elements) == 5:
+                    z0_left, z0_right, elev, displ, n_pts_geom = elements
+                    ltype = 2
+
                 # Only save data if it is supposed to be output
                 if ltype in [map_type_code, 2]:  # ltype=2 is combo line
                     if ltype == 0:
@@ -173,7 +181,6 @@ def _read_vector_map_combo(filename, crs, map_type):
                 # After reading all the points for the line,
                 # Set the header back to false and reset the point counter
                 if current_point_count == n_pts_geom:
-
                     if ltype in [map_type_code, 2]:
                         geometry = LineString(zip(x_pts, y_pts))
                         data["geometry"].append(geometry)
@@ -307,7 +314,6 @@ def vector_map_to_mapfile(gdf, filename, lctable=None, header="WindKit", mode="w
 
     map_type = "elevation"
     with open(filename, mode, newline="\r\n") as fobj:
-
         # File header
         if mode == "w":
             fobj.write(header + "\n")
@@ -556,13 +562,13 @@ def _crs_from_map_file(filename):
     f = open(filename, "r", newline="\r\n")
     header = f.readline()
     f.close()
-    regex_res = re.match(r"(.*)UTM-Proj.-(S|N).hemisph. Zone (\d*)(.*)", header)
 
-    if regex_res != None and len(regex_res.groups()) == 4:
-        band = regex_res[2]
-        utm = int(regex_res[3])
-        crs = _utmzone2epsg(utm, band)
-    else:
-        crs = None
+    epsg_lookup_table = _read_map_file_header_to_epsg_table()
+
+    crs = None
+    for string, epsg in epsg_lookup_table.items():
+        if string in header:
+            crs = epsg
+            break
 
     return crs
