@@ -115,8 +115,9 @@ def _has_wrg_header(infile, parse_header=False):
     return bool(match)
 
 
-def _infer_resource_file_nsec(infile):
-    """Infer the number of sectors in resource file
+def _infer_resource_file_nsec(infile, skip_first=True):
+    """Infer the number of sectors in resource file by reading
+    column 70-72 and converting it to an integer.
 
     Parameters
     ----------
@@ -134,15 +135,16 @@ def _infer_resource_file_nsec(infile):
         fobj = open(infile)
 
     # Skip the first line
-    fobj.readline()
+    if skip_first:
+        fobj.readline()
 
     # Read the second line
-    line = fobj.readline().strip()
+    line = fobj.readline()
 
     if not isinstance(infile, io.StringIO):
         fobj.close()
 
-    return (len(line.split()) - 9) // 3
+    return int(line[69:72])  # column 70-72 using python indexing
 
 
 def _read_resource_file(resource_file, crs, nsec=12, to_cuboid=False, **kwargs):
@@ -167,7 +169,7 @@ def _read_resource_file(resource_file, crs, nsec=12, to_cuboid=False, **kwargs):
     """
 
     has_wrg_header = _has_wrg_header(resource_file)
-    nsec = _infer_resource_file_nsec(resource_file)
+    nsec = _infer_resource_file_nsec(resource_file, skip_first=has_wrg_header)
 
     df = pd.read_fwf(
         resource_file,
@@ -283,6 +285,29 @@ def read_wrgfile(wrgfile, crs=None, to_cuboid=True, **kwargs):
     return _read_resource_file(wrgfile, crs=crs, to_cuboid=to_cuboid)
 
 
+def read_pwcfile(pwcfile, spatial_dataset):
+    """Reads .pwc predicted wind climate file from WAsP in XML format.
+       The .pwc file does not include spatial information, so it must
+       be passed as a parameter.
+
+    Parameters
+    ----------
+    pwcfile: str,pathlib.Path, io.StringIO
+        Path to .pwc file
+    spatial_dataset: xarray.Dataset
+        xarray dataset with the spatial info
+    """
+    ds = pd.read_xml(
+        pwcfile, names=["index", "sector", "sector_width", "wdfreq", "A", "k"]
+    )
+    ds = ds.set_index("sector")[["A", "k", "wdfreq"]].to_xarray()
+    ds = ds.assign_coords(create_sector_coords(ds.sector.size).coords)
+    ds = ds.expand_dims(spatial_dataset.dims)
+    ds = ds.assign_coords(spatial_dataset.coords)
+    wwc = update_var_attrs(ds, _WEIB_ATTRS)
+    return update_history(wwc)
+
+
 def _wrg_header(wwc):
     """Write WWC grid dimensions to WRG header with the format:
           nx ny xmin ymin cell_size
@@ -547,10 +572,10 @@ def weibull_combined(wwc):
     sum3 = sum3 / wwc["wdfreq"].sum("sector")
 
     aa = np.log(sum3) / 3.0 - np.log(sum1)
-    wwc["k_combined"] = _solve_k_vec(aa)
-    wwc["A_combined"] = sum1 / gamma(1.0 + 1.0 / wwc["k_combined"])
+    k_combined = _solve_k_vec(aa)
+    A_combined = sum1 / gamma(1.0 + 1.0 / k_combined)
 
-    return wwc["A_combined"], wwc["k_combined"]
+    return A_combined, k_combined
 
 
 def wwc_mean_windspeed(wwc, bysector=False, emergent=True):
@@ -571,11 +596,16 @@ def wwc_mean_windspeed(wwc, bysector=False, emergent=True):
     xarray.DataArray
         DataArray with the mean wind speed.
     """
+    if bysector and emergent:
+        raise ValueError(
+            "Emergent wind speed cannot be calculated for sectorwise wind speed."
+        )
+
     if emergent:
-        A, k = _get_A_k(wwc, bysector=False)
-        return (wwc["wdfreq"] * weibull_moment(A, k, 1)).sum("sector")
+        A, k = _get_A_k(wwc, bysector=True)  # emergent always use all sectors
+        return (wwc["wdfreq"] * weibull_moment(A, k, 1)).sum(dim="sector")
     else:
-        A, k = _get_A_k(wwc, bysector)
+        A, k = _get_A_k(wwc, bysector=bysector)
         return weibull_moment(A, k, 1)
 
 
@@ -598,9 +628,14 @@ def wwc_power_density(wwc, air_density, bysector=False, emergent=True):
     pd : xarray.DataArray
         Data array with power density.
     """
-    A, k = _get_A_k(wwc, bysector)
+    if bysector and emergent:
+        raise ValueError(
+            "Emergent power density cannot be calculated for sectorwise wind speed."
+        )
     if emergent:
-        pd = (air_density * wwc["wdfreq"] * weibull_moment(A, k, 3)).sum("sector")
+        A, k = _get_A_k(wwc, bysector=True)
+        pd = (air_density * wwc["wdfreq"] * weibull_moment(A, k, 3.0)).sum(dim="sector")
     else:
+        A, k = _get_A_k(wwc, bysector=bysector)
         pd = air_density * weibull_moment(A, k, 3)
     return 0.5 * pd
