@@ -6,6 +6,7 @@ Module containing a class and related methods for importing WAsP workspace
 XML and converting it to WindKit data structures.
 """
 import tempfile
+import warnings
 import zipfile
 from pathlib import Path
 
@@ -14,8 +15,9 @@ import numpy as np
 import pandas as pd
 from packaging import version
 
+from ._rvea_xml import _parse_rvea_generalised_mean_wind_climate
 from .binned_wind_climate import _parse_owc
-from .generalized_wind_climate import _parse_gwc
+from .generalized_wind_climate import _weibull_to_dataset
 from .metadata import _BWC_ATTRS, _WEIB_ATTRS, update_history, update_var_attrs
 from .spatial import add_crs, create_dataset
 from .vector_map import _crs_from_map_file, _read_vector_map_combo
@@ -155,7 +157,6 @@ class Workspace:
         search_template = '//*[@ID="{insert_id}"]'
         obj_cat = {}
         for i, elem in enumerate(self.root.iter("WaspHierarchyMember")):
-
             ID = elem.attrib["ID"].replace("{", "").replace("}", "")
             self._id_map.update({ID: i})
 
@@ -267,7 +268,8 @@ class Workspace:
                     crs = _crs_from_map_file(path_tmp.joinpath(info["filename"]))
                     self._map_files[ID][map_type]["crs"] = crs
                     self.catalogue[ID]["CRS"].update({map_type: crs})
-                    crs_ls += [crs]
+                    if crs is not None:
+                        crs_ls += [crs]
                 tempfolder.cleanup()
         crs_ls = np.unique(np.asarray(crs_ls))
         if len(crs_ls) == 1:
@@ -294,10 +296,14 @@ class Workspace:
                     "y-Location"
                 ]
             )
+
+            header = mast_xml.attrib["Description"]
+
             self.mast_coords = {
                 "height": height,
                 "west_east": west_east,
                 "south_north": south_north,
+                "header": header,
                 "crs": self.crs,
             }
 
@@ -394,7 +400,9 @@ class Workspace:
         ds = update_var_attrs(bwc, _BWC_ATTRS)
         return update_history(ds)
 
-    def get_gwc(self, ID, crs=None):
+    def get_gwc(
+        self, ID, crs=None, west_east=None, south_north=None, height=None, header=None
+    ):
         """
         Converts XML representation of gwc to WindKit gwc data structure.
 
@@ -404,31 +412,99 @@ class Workspace:
             ID of gwc object in .wwh file.
         crs : int, dict, str or pyproj.crs.CRS
             Value to initialize `pyproj.crs.CRS` Default, use from file.
+        west_east, south_north : float or None, optional
+            x- and y-coordinates of gwc location. Defaults to None, which means
+            the coordinates are taken from the met mast location or the file.
+            If the met mast location is not available, an error is raised.
+        height : float or None, optional
+            Height of gwc location. Defaults to None, which means the height
+            is taken from the met mast location or the file. If the met mast
+            location is not available, the height is set to 0 m.
+        header : str or None, optional
+            Header of gwc. Defaults to None, which means the header
+            is attempted to be taken from the met mast or the file.
+
         Returns
         -------
         xarray.Dataset
             xarray Dataset that is formatted to match the gwc description.
+
+        Raises
+        ------
+        ValueError
+            If the ID does not exist in the catalogue.
+        ValueError
+            If the ID is not linked to a gwc.
+        ValueError
+            If the mast location is not available and the coordinates are not
+            provided.
+        ValueError
+            If no CRS is provided and the file does not contain a CRS.
+
+        Warns
+        -----
+        UserWarning
+            If no height is provided and the mast location is not available.
+            The height is set to 0 m.
+
         """
         if ID not in self.catalogue:
             raise ValueError("ID does not exist in object catalogue")
         if self.catalogue[ID]["ObjectDescription"] != "Generalised wind climate":
             raise ValueError("ID is not linked to generalised wind climate")
+
+        # subset XML
+        tree = self._get_object_xml(ID)
+
+        gwc_data = _parse_rvea_generalised_mean_wind_climate(
+            tree.find(".//RveaGeneralisedMeanWindClimate")
+        )
+
+        if west_east is not None and south_north is not None:
+            if crs is None:
+                warnings.warn("No CRS provided, defaulting to file CRS")
+                crs = self.crs
+        elif self.mast_coords is not None:
+            west_east = self.mast_coords["west_east"]
+            south_north = self.mast_coords["south_north"]
+            height = self.mast_coords["height"]
+            crs = self.mast_coords["crs"]
+            if header is None:
+                header = self.mast_coords["header"]
+        elif "longitude" in gwc_data and "latitude" in gwc_data:
+            west_east = gwc_data["longitude"]
+            south_north = gwc_data["latitude"]
+            crs = "EPSG:4326"
+        else:
+            raise ValueError(
+                "No location information provided, either provide coordinates or mast location"
+            )
+
+        if height is None:
+            height = 0.0
+            warnings.warn("No height provided, defaulting to 0 m")
+
         if self.crs == None and crs == None:
             raise ValueError(
                 "No CRS associated with generalised wind climate, CRS must be provided!"
             )
-        if crs == None and self.mast_coords["crs"] != None:
-            crs = self.mast_coords["crs"]
 
-        # subset XML
-        gwc = _parse_gwc(self._get_object_xml(ID))
+        if header is None:
+            header = ""
+
+        gwc = _weibull_to_dataset(
+            wdfreq=gwc_data["wdfreq"],
+            A=gwc_data["A"],
+            k=gwc_data["k"],
+            gen_roughness=gwc_data["gen_roughness"],
+            gen_height=gwc_data["gen_height"],
+            west_east=west_east,
+            south_north=south_north,
+            height=height,
+            crs=crs,
+            wasp_header=header,
+        )
 
-        # adding mast location and crs
-        if self.mast_coords != None:
-            gwc.height.values = np.array([self.mast_coords["height"]])
-            gwc.south_north.values = np.array([self.mast_coords["south_north"]])
-            gwc.west_east.values = np.array([self.mast_coords["west_east"]])
-            return add_crs(gwc, crs=self.crs)
         gwc = add_crs(gwc, crs)
         gwc = update_var_attrs(gwc, _WEIB_ATTRS)
         return update_history(gwc)
